
#include "GMMFactory.hpp"

namespace SPN {

	GMMFactory::GMMFactory() {
		numDims_ = 10;
		numComponents_ = 5;
	}

	GMMFactory::~GMMFactory() {
	}

	void GMMFactory::setNumDims(unsigned D) {
		numDims_ = D;
	}

	void GMMFactory::setNumComponents(unsigned K) {
		numComponents_ = K;
	}

	Network GMMFactory::makeNetwork() {
		Network network;
		makeNetwork(network);
		return network;
	}

	void GMMFactory::makeNetwork(Network &network) {
		ValLayer *inputLayer = (ValLayer*)generateLayer(Layer::VAL_LAYER, numDims_);
		GaussianLayer *gaussLayer = (GaussianLayer*)generateLayer(Layer::GAUSSIAN_LAYER, numDims_ * numComponents_);
		ProdLayer *prodLayer = (ProdLayer*)generateLayer(Layer::PROD_LAYER, numComponents_);
		SumLayer *sumLayer = (SumLayer*)generateLayer(Layer::SUM_LAYER, 1);

		unsigned gaussCounter = 0;
		for (unsigned d = 0; d < numDims_; d++) {
			for (unsigned k = 0; k < numComponents_; k++) {
				(*gaussLayer)(gaussCounter)->connectChild((*inputLayer)(d));
				(*prodLayer)(k)->connectChild((*gaussLayer)(gaussCounter));
				if (d == 0)
					(*sumLayer)(0)->connectChild((*prodLayer)(k));
				gaussCounter++;
			}
		}
		network.addLayer(inputLayer);
		network.addLayer(gaussLayer);
		network.addLayer(prodLayer);
		network.addLayer(sumLayer);
	}

}
