/*
 *
 * Class for generating SPN objects: nodes, layers and networks
 * Main purpose: take care about memory allocation
 *
 */

#ifndef NETWORKFACTORY_HPP
#define	NETWORKFACTORY_HPP

#include "Network.hpp"

namespace SPN {

	class NetworkFactory {
	private:
		UMap<NetworkId, Network*> generatedNetworkPtrs_;
		UMap<LayerId, Layer*> generatedLayerPtrs_;
		UMap<NodeId, Node* > generatedNodePtrs_;
		
	public:
		NetworkFactory();
		virtual ~NetworkFactory();
		
		Network* generateNetwork();
		void deleteNetwork(Network *network);
		Layer* generateLayer(LayerType type, size_t numNodes);
		void deleteLayer(Layer *layer);
		NodeVec generateNodes(NodeType type, size_t numNodes);
		Node* generateNode(NodeType type);
		void deleteNode(Node *node);

		void clear();
	};

}

#endif	/* NETWORKFACTORY_HPP */

