
#include "Node.hpp"

namespace SPN {

	ProdNode::ProdNode() : Node(PROD_NODE) {
		logVal_ = 0.0;
		zeroChildFlag_ = 0;
	}

	ProdNode::~ProdNode() {
	}

	void ProdNode::eval() {
		logVal_ = 0.0;
		zeroChildFlag_ = 0;

		for (unsigned k = 0; k < getNumChildren(); k++) {
			double childLogVal = getChild(k)->getLogVal();
			if (isnan(childLogVal))
				logVal_ = nan;
			else if (childLogVal > -inf)
				logVal_ += childLogVal;
			else {
				zeroChildFlag_++;
				if (zeroChildFlag_ == 2) {
					logVal_ = -inf;
					break;
				}
			}
		}
	}

	double ProdNode::getLogVal() const {
		if (zeroChildFlag_ == 0)
			return logVal_;
		else
			return -inf;
	}

	double ProdNode::getVal() const {
		if (zeroChildFlag_ == 0)
			return exp(logVal_);
		else
			return 0;
	}

	double ProdNode::getLog_Dval_Dchild(Node *child) {
		if (!isChild(child))
			throw std::runtime_error(" ProdNode::getLog_Dval_Dchild(Node *child): child node not found");

		if (zeroChildFlag_ == 0)
			return logVal_ - child->getLogVal();
		else if ((zeroChildFlag_ == 1) && (!(child->getLogVal() > -inf)))
			return logVal_;
		else
			return -inf;
	}

}

