
#include "DataSource.hpp"
#include "Network.hpp"
#include "NetworkFactory.hpp"
#include "FileIO.hpp"
#include <string>
#include <iostream>
#include <math.h>
#include "Utils.hpp"

using namespace SPN;
using namespace std;


int main(int argc, char **argv) {
	NetworkFactory factory;
	Network spn;
	DataSource dataSource, valDataSource;

	string fileName;
	stringstream convert;
	string useText =
		string("Infer the log-likelihood of a sum-product network\n") +
		string("Useage: ") + string(argv[0]) + string(" modelfile datafile\n") +
		string("\n") +
		string("Parameters: \n") +
		string("\n") +
		string("mode                                        = 1 \n") +
		string("   mode = 1: normalized LLs \n") +
		string("   mode = 2: unnormalized LLs \n") +
		string("   mode = 3: log-normalization constant \n") +		
		string("upwardsPass (max, sum)                      = sum \n") +
		string("outputfile                                  = \"\" \n") +
		string("   outputfile = \"\": write to cout \n") +
		string("\n");

	string outputfilename = "";
	unsigned mode = 1;	
	string upwardsPass = "sum";


	if (argc < 3) {
		std::cout << useText << std::endl;
		return 0;
	}

	map<string, string> optionMap;
	map<string, string>::iterator it;
	stringstream inputStream;

	if (!parseOptions(argc, argv, 3, optionMap)) {
		std::cout << useText;
		return -1;
	}
	else {
		for (it = optionMap.begin(); it != optionMap.end(); it++) {
			if (it->first == "mode") {
				inputStream.clear();
				inputStream.str(it->second);
				inputStream >> mode;
				if (inputStream.fail()) {
					std::cout << useText;
					return -1;
				}
			}			
			else if (it->first == "upwardsPass") {
				upwardsPass = it->second;
				if ((upwardsPass != "sum") && (upwardsPass != "max")) {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "outputfile") {
				outputfilename = it->second;
			}
			else {
				std::cout << useText;
				return -1;
			}
		}
	}

	if (mode > 3) {
		std::cout << useText;
		return -1;
	}

	//if (outputfilename != "") {
	//	cout << "mode                  = " << mode << endl;
    //	cout << "outputfile            = " << outputfilename << endl;		
	//	cout << "upwardsPass           = " << upwardsPass << endl;
	//}		
	
	try {
		loadNetwork(&spn, argv[1], factory);
		dataSource.readFile(argv[2]);
	}
	catch (std::exception &ex) {
		std::cout << ex.what() << std::endl;
		return -1;
	}

	if (dataSource.getNumSamples() == 0)
		return 0;

	std::ofstream outputFile;
	if (outputfilename != "") {
		outputFile.open(outputfilename.c_str());
		if (!outputFile.is_open()) {
			std::cout << "unable to open file" << std::endl;
			return -1;
		}
	}

	if (outputfilename != "") {
		outputFile << "SPN_DATA:txt" << std::endl;
		outputFile << 1 << std::endl;
		if (mode == 3)
			outputFile << 1 << std::endl;
		else
			outputFile << dataSource.getNumSamples() << std::endl;
	}
	else {
		std::cout << "SPN_DATA:txt" << std::endl;
		std::cout << 1 << std::endl;
		if (mode == 3)
			std::cout << 1 << std::endl;
		else
			std::cout << dataSource.getNumSamples() << std::endl;
	}

	DoubleVec sample;
	double logZ = -SPN::inf;

	if (dataSource.getNumSamples() > 0) {
		sample.resize(dataSource.getSample(0).size());
		for (unsigned k = 0; k < sample.size(); k++)
			sample[k] = SPN::nan;
		spn.setVals(sample);
		spn.eval();

		if (spn.getLayer(spn.getNumLayers() - 1)->getNumNodes() != 1)
			throw (std::logic_error("highest layer has to contain exactly one node"));

		logZ = spn.getLayer(spn.getNumLayers() - 1)->getNode(0)->getLogVal();
	}

	if (mode == 3) {
		if (outputfilename != "")
			outputFile << logZ << std::endl;
		else
			std::cout << logZ << std::endl;
	}
	else {

		if (upwardsPass == "max")
			spn.setMaxMode();
		else
			spn.setSumMode();

		for (unsigned sIdx = 0; sIdx < (unsigned)dataSource.getNumSamples(); sIdx++) {
			sample = dataSource(sIdx);

			spn.setVals(sample);
			spn.eval();

			if (spn.getLayer(spn.getNumLayers() - 1)->getNumNodes() != 1)
				throw (std::logic_error("highest layer has to contain exactly one node"));

			double tmpval;
			if (mode == 1)
				tmpval = spn.getLayer(spn.getNumLayers() - 1)->getNode(0)->getLogVal() - logZ;
			else
				tmpval = spn.getLayer(spn.getNumLayers() - 1)->getNode(0)->getLogVal();

			if (outputfilename != "") 
				outputFile << tmpval << " " << std::endl;
			else 
				std::cout << tmpval << " " << std::endl;
		}
	}

	if (outputfilename != "")
		outputFile.close();
}

