
#include "Network.hpp"
#include "DataSource.hpp"
#include "PDLearning.hpp"
#include "Utils.hpp"
#include "FileIO.hpp"
#include <iostream>
#include <string>

using namespace SPN;
using namespace std;

int main(int argc, char** argv) {

	string useText =
		string("Trains a sum-product network in Poon & Domingos style\n") +
		string("Useage: ") + string(argv[0]) + string(" datafile outputfile\n") +
		string("\n") +
		string("Parameters: \n") +
		string("\n") +
		string("valDataFile                 = "" \n") +
		string("width                       = 64 \n") +
		string("height                      = 64 \n") +
		string("coarseRes                   = [4] (format [x,y,z] without spaces)\n") +
		string("coarseResX                  = [] \n") +
		string("coarseResY                  = [] \n") +
		string("batchSize                   = 50 \n") +
		string("numSums                     = 20 \n") +
		string("numGauss                    = 4 \n") +
		string("gaussMeansFile              = '' \n") +
		string("numIter                     = 30 \n") +
		string("stop_absLikelihoodChange    = 0.1 \n") +
		string("stop_relLikelihoodChange    = -1e21 \n") +
		string("earlyStoppingKrel           = 3 \n") +
		string("earlyStoppingKabs           = 5 \n") +
		string("sparsePrior                 = 1 \n") +
		string("sparsePriorAnneal           = 10 \n") +
		string("preSubtractCounts           = 1\n") +
		string("flatEM                      = 1\n") +
		string("numChannels                 = 1 \n") +
		string("pixelOrder (RM, CM)         = RM \n") +
		string("\n");

	int width = 64;
	int height = 64;
	UnsignedVec coarseRes;
	UnsignedVec coarseResX;
	UnsignedVec coarseResY;
	coarseRes.resize(1);
	coarseRes[0] = 4;
	coarseResX.clear();
	coarseResY.clear();
	int incrEM_batchSize = 50;
	int numSumsPerRectangle = 20;
	int numGauss = 4;
	int maxIter = 30;
	double stopping_objMinAbsChange = 0.1;
	double stopping_objMinRelChange = -1e21;
	unsigned earlyStoppingKabs = 5;
	unsigned earlyStoppingKrel = 3;
	double sparsePrior = 1.0;
	int sparsePriorAnneal = 10;
	bool preSubtract = true;
	bool flatEM = true;
	int numChannels = 1;
	int pixelOrder = PDLearning::ROW_MAJOR;
	string valDataFile = "";
	string gaussMeansFile = "";

	map<string, string> optionMap;
	map<string, string>::iterator it;
	stringstream inputStream;


	if (argc < 3) {
		std::cout << useText;
		return -1;
	}

	if (!parseOptions(argc, argv, 3, optionMap)) {
		std::cout << useText;
		return -1;
	}
	else {
		for (it = optionMap.begin(); it != optionMap.end(); it++) {
			if (it->first == "width") {
				inputStream.clear();
				inputStream.str(it->second);
				inputStream >> width;
				if (inputStream.fail()) {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "height") {
				inputStream.clear();
				inputStream.str(it->second);
				inputStream >> height;
				if (inputStream.fail()) {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "coarseRes") {
				if (it->second.size() < 2) {
					std::cout << useText;
					return -1;
				}

				string tmp = it->second.substr(1, it->second.size() - 2);
				size_t pos;
				unsigned val;
				coarseRes.clear();

				while (tmp.size() > 0) {
					inputStream.clear();
					pos = tmp.find_first_of(",");
					if (pos == string::npos) {
						inputStream.str(tmp);
						tmp = "";
					}
					else {
						inputStream.str(tmp.substr(0, pos));
						tmp = tmp.substr(pos + 1, tmp.size() - pos);
					}
					inputStream >> val;
					if (inputStream.fail()) {
						std::cout << useText;
						return -1;
					}

					coarseRes.push_back(val);
				}
			}
			else if (it->first == "coarseResX") {
				if (it->second.size() < 2) {
					std::cout << useText;
					return -1;
				}

				string tmp = it->second.substr(1, it->second.size() - 2);
				size_t pos;
				unsigned val;
				coarseResX.clear();

				while (tmp.size() > 0) {
					inputStream.clear();
					pos = tmp.find_first_of(",");
					if (pos == string::npos) {
						inputStream.str(tmp);
						tmp = "";
					}
					else {
						inputStream.str(tmp.substr(0, pos));
						tmp = tmp.substr(pos + 1, tmp.size() - pos);
					}
					inputStream >> val;
					if (inputStream.fail()) {
						std::cout << useText;
						return -1;
					}

					coarseResX.push_back(val);
				}
			}
			else if (it->first == "coarseResY") {
				if (it->second.size() < 2) {
					std::cout << useText;
					return -1;
				}

				string tmp = it->second.substr(1, it->second.size() - 2);
				size_t pos;
				unsigned val;
				coarseResY.clear();

				while (tmp.size() > 0) {
					inputStream.clear();
					pos = tmp.find_first_of(",");
					if (pos == string::npos) {
						inputStream.str(tmp);
						tmp = "";
					}
					else {
						inputStream.str(tmp.substr(0, pos));
						tmp = tmp.substr(pos + 1, tmp.size() - pos);
					}
					inputStream >> val;
					if (inputStream.fail()) {
						std::cout << useText;
						return -1;
					}

					coarseResY.push_back(val);
				}
			}
			else if (it->first == "batchSize") {
				inputStream.clear();
				inputStream.str(it->second);
				inputStream >> incrEM_batchSize;
				if (inputStream.fail()) {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "numSums") {
				inputStream.clear();
				inputStream.str(it->second);
				inputStream >> numSumsPerRectangle;
				if (inputStream.fail()) {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "numGauss") {
				inputStream.clear();
				inputStream.str(it->second);
				inputStream >> numGauss;
				if (inputStream.fail()) {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "gaussMeansFile") {
				gaussMeansFile = it->second;
			}
			else if (it->first == "numIter") {
				inputStream.clear();
				inputStream.str(it->second);
				inputStream >> maxIter;
				if (inputStream.fail()) {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "stop_absLikelihoodChange") {
				inputStream.clear();
				inputStream.str(it->second);
				inputStream >> stopping_objMinAbsChange;
				if (inputStream.fail()) {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "stop_relLikelihoodChange") {
				inputStream.clear();
				inputStream.str(it->second);
				inputStream >> stopping_objMinRelChange;
				if (inputStream.fail()) {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "earlyStoppingKrel") {
				inputStream.clear();
				inputStream.str(it->second);
				inputStream >> earlyStoppingKrel;
				if (inputStream.fail()) {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "earlyStoppingKabs") {
				inputStream.clear();
				inputStream.str(it->second);
				inputStream >> earlyStoppingKabs;
				if (inputStream.fail()) {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "sparsePrior") {
				inputStream.clear();
				inputStream.str(it->second);
				inputStream >> sparsePrior;
				if (inputStream.fail()) {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "sparsePriorAnneal") {
				inputStream.clear();
				inputStream.str(it->second);
				inputStream >> sparsePriorAnneal;
				if (inputStream.fail()) {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "preSubtractCounts") {
				inputStream.clear();
				inputStream.str(it->second);
				inputStream >> preSubtract;
				if (inputStream.fail()) {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "flatEM") {
				inputStream.clear();
				inputStream.str(it->second);
				inputStream >> flatEM;
				if (inputStream.fail()) {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "numChannels") {
				inputStream.clear();
				inputStream.str(it->second);
				inputStream >> numChannels;
				if (inputStream.fail()) {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "pixelOrder") {
				if (it->second == "RM")
					pixelOrder = PDLearning::ROW_MAJOR;
				else if (it->second == "CM")
					pixelOrder = PDLearning::COLUMN_MAJOR;
				else {
					std::cout << useText;
					return -1;
				}
			}
			else if (it->first == "valDataFile") {
				valDataFile = it->second;
			}
			else {
				std::cout << useText;
				return -1;
			}
		}
	}

	cout << "valDataFile              = " << valDataFile << endl;
	cout << "width                    = " << width << endl;
	cout << "height                   = " << height << endl;
	cout << "coarseRes                = [";
	for (unsigned k = 0; k < coarseRes.size(); k++) {
		cout << coarseRes[k];
		if (k + 1 < coarseRes.size())
			std::cout << ",";
	}
	cout << "]" << endl;
	cout << "coarseResX               = [";
	for (unsigned k = 0; k < coarseResX.size(); k++) {
		cout << coarseResX[k];
		if (k + 1 < coarseResX.size())
			std::cout << ",";
	}
	cout << "]" << endl;
	cout << "coarseResY               = [";
	for (unsigned k = 0; k < coarseResY.size(); k++) {
		cout << coarseResY[k];
		if (k + 1 < coarseResY.size())
			std::cout << ",";
	}
	cout << "]" << endl;
	cout << "batchSize                = " << incrEM_batchSize << endl;
	cout << "numSums                  = " << numSumsPerRectangle << endl;
	cout << "numGauss                 = " << numGauss << endl;
	cout << "gaussMeansFile           = " << gaussMeansFile << endl;
	cout << "numIter                  = " << maxIter << endl;
	cout << "stop_absLikelihoodChange = " << stopping_objMinAbsChange << endl;
	cout << "stop_relLikelihoodChange = " << stopping_objMinRelChange << endl;
	cout << "earlyStoppingKrel        = " << earlyStoppingKrel << endl;
	cout << "earlyStoppingKabs        = " << earlyStoppingKabs << endl;
	cout << "sparsePrior              = " << sparsePrior << endl;
	cout << "sparsePriorAnneal        = " << sparsePriorAnneal << endl;
	cout << "preSubtractCounts        = " << preSubtract << endl;
	cout << "flatEM                   = " << flatEM << endl;
	cout << "numChannels              = " << numChannels << endl;
	if (pixelOrder == PDLearning::ROW_MAJOR)
		cout << "pixelOrder               = RM" << endl;
	else
		cout << "pixelOrder               = CM" << endl;
	cout << endl;

	try {
		PDLearning learner;
		NetworkFactory factory;

		DataSource dataSource, valDataSource;
		DataSource gaussMeansDataSource;
		std::vector<std::vector<double> > gaussMeans;
		gaussMeans.clear();

		// load data
		dataSource.readFile(argv[1]);
		if (valDataFile != "")
			valDataSource.readFile(valDataFile);
		//dataSource.attachFile(argv[1]);
		//if (valDataFile != "")
		//  valDataSource.attachFile(valDataFile);

		if (gaussMeansFile != "") {
			gaussMeansDataSource.readFile(gaussMeansFile);
			gaussMeansDataSource.makeDataCopy(gaussMeans);
		}

		if (valDataFile != "")
			learner.options.validationDataSource = &valDataSource;
		else
			learner.options.validationDataSource = NULL;
		
		learner.setDims(width, height);
		learner.setNumChannels(numChannels);
		if (pixelOrder == PDLearning::ROW_MAJOR)
			learner.setRowMajor();
		else
			learner.setColumnMajor();
		learner.setCoarseRes(coarseRes);
		learner.setCoarseResX(coarseResX);
		learner.setCoarseResY(coarseResY);
		learner.setMaxNumSumsPerRectangle(numSumsPerRectangle);
		learner.setNumGaussPerPixel(numGauss);
		learner.setGaussMeans(gaussMeans);
		learner.options.maxIter = maxIter;
		learner.options.sparsePrior = sparsePrior;
		learner.options.sparsePriorAnneal = sparsePriorAnneal;
		learner.options.incrEM_batchSize = incrEM_batchSize;
		learner.options.stopping_objMinAbsChange = stopping_objMinAbsChange;
		learner.options.stopping_objMinRelChange = stopping_objMinRelChange;
		learner.options.earlyStoppingKrel = earlyStoppingKrel;
		learner.preSubtract(preSubtract);
		learner.flatEM(flatEM);

		/////////////////////////////////////////////////////
		cout << "Run SPN training" << endl;
		Network* spn = learner.learn(&dataSource, factory);
		/////////////////////////////////////////////////////

		spn->performStructureCheck();
		if (spn->isFeedforward())
			std::cout << "Feedforward " << std::endl;
		else
			std::cout << "not Feedforward " << std::endl;

		if (spn->isComplete())
			std::cout << "Complete " << std::endl;
		else
			std::cout << "not Complete " << std::endl;

		if (spn->isDecomposable())
			std::cout << "Decomposable " << std::endl;
		else
			std::cout << "not Decomposable " << std::endl;

		std::cout << spn->getNumLayers() << std::endl;
		std::cout << spn->getNumGaussians() << std::endl;
		std::cout << spn->getNumSums() << std::endl;
		std::cout << spn->getNumProds() << std::endl;
		std::cout << spn->getNumWeights() << std::endl;
		std::cout << spn->getLayer(spn->getNumLayers() - 1)->getNumNodes() << std::endl;

		saveNetwork(spn, argv[2]);
	}
	catch (std::exception& e) { std::cout << e.what() << '\n'; }
		
}
