function [C, idx, ObjHist] = LBGcluster(X, K, minRelImprov)

if nargin < 3
    minRelImprov = [];
end

[~,N] = size(X);

Ktmp = 1;
C = mean(X, 2);

Range = max(X, [], 2) - min(X, [], 2);
epsilon = 1e-12;

ObjHist = [];

if ~isempty(minRelImprov)
   oldOveralObj = inf;
end

while Ktmp < K
    
    if ~isempty(minRelImprov) && Ktmp > 1
        fprintf('LBG cluster: current K = %d   oldObj = %d   newObj = %d  rel = %d\n',...
            size(C,2), ...
            oldOveralObj, ...
            ObjHist(end), ...
            (oldOveralObj - ObjHist(end)) / (Knew*ObjHist(end)));
        
        if (oldOveralObj - ObjHist(end)) / (Knew*ObjHist(end)) < minRelImprov
            break
        end
        oldOveralObj = ObjHist(end);
    end
    
    if Ktmp*2 <= K
        Knew = Ktmp;
    else
        Knew = K - Ktmp;
    end
    
    rp = randperm(Ktmp);
    Cnew = C(:,rp(1:Knew));
    
    %%% distortion
    Distortion = epsilon * (rand(size(Cnew)) - 0.5) .* repmat(Range,1,Knew);
    Cnew = Cnew + Distortion;
    C(:,rp(1:Knew)) = C(:,rp(1:Knew)) - Distortion;
    
    C = [C, Cnew];
    Ktmp = size(C,2);
    
    %%% run k-means
    oldObj = inf;
    while 1
        E = zeros(Ktmp, N);
        for c = 1:Ktmp
            E(c,:) = sum((X - repmat(C(:,c), 1, N)).^2);
        end
        
        [minE, minIdx] = min(E,[],1);
        
        Obj = sum(minE);
        if all((oldObj - Obj) / Obj < 1e-15)
            break;
        end
        oldObj = Obj;
        ObjHist = [ObjHist; Obj];
        
        for c = 1:Ktmp
            C(:,c) = mean(X(:,minIdx == c),2);
        end
 
    end
end

if ~isempty(minRelImprov) 
    K = size(C,2);
end

E = zeros(K, N);
for c = 1:K
    E(c,:) = sum((X - repmat(C(:,c), 1, N)).^2);
end

[~, idx] = min(E,[],1);

    