function [d, D] = LSD(xhat, x, fstart, fstop, fs, winMS, overlapMS)

if nargin < 3
    fstart = 0;
end

if nargin < 4
    fstop = 1;
end

if nargin < 5
    fs = 16000;
end

if nargin < 6
    winMS = 50;
end

if nargin < 7
    overlapMS = 25;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[xhat,x] = makeSameLength(xhat, x, 0);
 
winLen = round(fs * winMS / 1000);
overlap = round(fs * overlapMS / 1000);
FFTlen = 2^nextpow2(winLen);

Om = (0:(FFTlen/2)-1) * (2*pi/(FFTlen));


Bhat = buffer(xhat, winLen, overlap,'nodelay');
B = buffer(x, winLen, overlap,'nodelay');

ahat = lpc(Bhat, 9);
a = lpc(B,9);

Ehat = zeros(FFTlen/2, size(Bhat,2));
E    = zeros(FFTlen/2, size(B,2));


for k=1:size(ahat,2)
    Ehat = Ehat + exp(-1j * (k-1) * Om)' * ahat(:,k)';
end
Ehat = repmat(std(Bhat, [], 1), FFTlen/2, 1) ./ abs(Ehat);


for k=1:size(a,2)
    E = E + exp(-1j * (k-1) * Om)' * a(:,k)';
end
E = repmat(std(B, [], 1),    FFTlen/2, 1) ./ abs(E);


start = round(size(Ehat,1) * fstart) + 1;
stop  = round(size(Ehat,1) * fstop);

D =  (2 / FFTlen) * sum(( 20 * log10(E(start:stop,:)) - 20 * log10(Ehat(start:stop,:)) ).^2, 1);
d = mean(sqrt(D));

