
clear all

setPaths

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Speaker dependent data %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

speakerLabels = [1,2,18,20];
numSpeakers = length(speakerLabels);

%%% Signal parameters
fsWork     = 16000;
frameLen   = 512;
numOverlap = 384;
win = hamming(frameLen);

maxNumSamples = 75000;

for sCount = 1:numSpeakers    
    speaker = speakerLabels(sCount);
    fprintf('Speaker: %d\n',speaker);
    
    %%% Load Data
    x = [];
    fid = fopen(sprintf('FileLists/Train%d.txt', speakerLabels(sCount)));
    while 1
        inputline = fgetl(fid);
        if ~ischar(inputline) || isempty(inputline)
            break
        end
        
        %%% read sound file
        [xInput,fsInput]= audioread([gridPath,int2str(speakerLabels(sCount)),'/',inputline]);
        if fsInput ~= fsWork
            %%% resample, subtract mean and normalize
            xInput = resample(xInput, fsWork, fsInput);
        end
        xInput = xInput - mean(xInput);
        xInput = xInput / std(xInput);
        x = [x; xInput];        
    end
    fclose(fid);
    
    %%% Spectrogram
    Data = log(abs(spectrogram(x, win, numOverlap, frameLen)));
    Data = Data(:,1:maxNumSamples)';
       
    MU = mean(Data,1);
    SIGMA = std(Data,[],1);
    
    Data = Data - repmat(MU, size(Data,1), 1);
    Data = Data ./ repmat(SIGMA, size(Data,1), 1);
    
    writeSPNVecList([dataPath, 'TrainSpeaker', int2str(speakerLabels(sCount)), '.dat'], Data);
    save([dataPath, 'TrainSpeaker', int2str(speakerLabels(sCount)), '.mat'], 'Data');
    save([dataPath, 'PreprocessSpeaker', int2str(speakerLabels(sCount)), '.mat'], 'MU', 'SIGMA');
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Speaker independent data %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('Speaker independent\n');

speakerLabels = setdiff(1:34, [1,2,18,20]);
numSpeakers = length(speakerLabels);

numSamplesOfSpeaker = floor(maxNumSamples / numSpeakers) * ones(numSpeakers, 1);
numSamplesOfSpeaker(1:mod(maxNumSamples, numSpeakers)) = numSamplesOfSpeaker(1:mod(maxNumSamples, numSpeakers))+1;

Data = [];
for sCount = 1:numSpeakers
    x = [];
    fid = fopen(sprintf('FileLists/Other/Train%d.txt', speakerLabels(sCount)));
    while 1
        inputline = fgetl(fid);
        if ~ischar(inputline) || isempty(inputline)
            warning('too little data')
            break
        end
        
        %%% read sound file
        [xInput,fsInput]= audioread([gridPath,int2str(speakerLabels(sCount)),'/',inputline]);
        if fsInput ~= fsWork
            %%% resample, subtract mean and normalize
            xInput = resample(xInput, fsWork, fsInput);
            xInput = xInput - mean(xInput);
            xInput = xInput / std(xInput);
            x = [x; xInput];
        end
        
        if length(x) > numSamplesOfSpeaker(sCount) * (frameLen-numOverlap) + 3*512
            break
        end
    end
    fclose(fid);
    
    %%% Spectrogram
    Spec = log(abs(spectrogram(x, win, numOverlap, frameLen)));
    Spec = Spec(:,1:numSamplesOfSpeaker(sCount));
    Data = [Data; Spec'];
end

%%% normalize
MU = mean(Data,1);
SIGMA = std(Data,[],1);

Data = Data - repmat(MU, size(Data,1), 1);
Data = Data ./ repmat(SIGMA, size(Data,1), 1);

writeSPNVecList([dataPath, 'TrainSpeakerIndependent.dat'], Data);
save([dataPath, 'TrainSpeakerIndependent.mat'], 'Data');
save([dataPath, 'PreprocessSpeakerIndependent.mat'], 'MU', 'SIGMA', 'numSamplesOfSpeaker');


