
setPaths

%%% Signal parameters
fsWork     = 16000;
frameLen   = 512;
numOverlap = 384;
win = hamming(frameLen);

width = 1;
lookAhead = 3;
numHMMstates = 64;

%%% SD = 0 <=> Speaker independent
%%% SD = 1 <=> Speaker dependent
SD = 1;

speakerLabels = [1,2,18,20];
numSpeakers = length(speakerLabels);

load('telephone.mat');
[~,telephoneDelay] = max(abs(telephone.numerator));

for sCount = 1:numSpeakers
    speaker = speakerLabels(sCount);
    fprintf('Speaker: %d\n',speaker);
    
    if SD
        dir = sprintf('%sSPN_HMM_K%d/SD/speaker%d/', resultPath, numHMMstates, speaker);
    else
        dir = sprintf('%sSPN_HMM_K%d/SI/speaker%d/', resultPath, numHMMstates, speaker);
    end
    [SUCCESS, MESS] = mkdir(dir);
    
    %%% Load Data
    clear XTEST
    clear FILENAMES
    
    count = 0;
    fid = fopen(sprintf('FileLists/Test%d.txt', speakerLabels(sCount)));
    while 1
        inputline = fgetl(fid);
        if ~ischar(inputline) || isempty(inputline)
            break
        end
        
        count = count + 1;
        
        %%% read sound file
        [xInput,fsInput]= audioread([gridPath,int2str(speakerLabels(sCount)),'/',inputline]);
        
        %%% resample, subtract mean and normalize
        if fsInput ~= fsWork
            xInput = resample(xInput, fsWork, fsInput);
        end
        xInput = xInput - mean(xInput);
        xInput = xInput / std(xInput);
        XTEST{count} = xInput;
        FILENAMES{count} = inputline;
    end
    fclose(fid);
    
    for k = 1:length(XTEST)
        fileName = [dir, FILENAMES{k}];
        if exist(fileName, 'file')
            continue
        end
        tmp = 'tmp';
        save(fileName, 'tmp');
        
        xTrue = XTEST{k};
        XTRUE = log(abs(spectrogram(xTrue, win, numOverlap, frameLen)));
        
        %%% telephone filter
        x = conv(xTrue,telephone.numerator);
        x = x(telephoneDelay:telephoneDelay+length(xTrue)-1);
        
        %%% Spectrogram
        Xcompl = spectrogram(x, win, numOverlap, frameLen);
        X = log(abs(Xcompl));
        phase = angle(Xcompl);
        
        Xmiss = X;
        Xmiss(1:2,:) = nan;
        Xmiss(126:end,:) = nan;
        
        [D, L] = size(X);
        
        Data = zeros(L, D * width);
        
        for s = 1:L
            if s >= width
                Xtmp = Xmiss(:, s-width+1:s);
            else
                Xtmp = Xmiss(:, 1:s);
                Xtmp = [nan(D, width-s), Xtmp];
            end
            Xtmp = Xtmp';
            Data(s,:) = Xtmp(:);
        end
        
        %%% inference
        if SD
            load([dataPath, 'PreprocessSpeaker',int2str(speaker),'.mat'])
        else
            load([dataPath, 'PreprocessSpeakerIndependent.mat'])
        end
        
        Data = Data - repmat(MU, size(Data,1), 1);
        Data = Data ./ repmat(SIGMA, size(Data,1), 1);
        
        N = size(Data,1);
        
        %%% likelihood
        likelihood = zeros(numHMMstates, N);
        for kc = 1:numHMMstates
            if SD
                modelname = sprintf('%sSPNHMM/Speaker%d/SPN_K%d_%d.mod', modelPath, speaker, numHMMstates, kc);
            else
                modelname = sprintf('%sSPNHMM/SpeakerIndependent/SPN_K%d_%d.mod', modelPath, numHMMstates, kc);
            end
            
            likelihood(kc, :) = inferLL(Data,[],...
                'binPath', binPath,...
                'modelFile', modelname);
        end
        
        clear P
        clear T
        if SD
            load(sprintf('%sSPNHMM/Speaker%d/HMM_K%d.mat', modelPath, speaker, numHMMstates));
        else
            load(sprintf('%sSPNHMM/SpeakerIndependent/HMM_K%d.mat', modelPath, numHMMstates));
        end
        
        %%% Forward-Backward
        Joint = zeros(numHMMstates, N);
        alpha = zeros(numHMMstates, N);
        beta = zeros(numHMMstates, N);
        
        alpha(:,1) = log(P);
        for n = 2:N
            Joint(:,n-1) = alpha(:,n-1) + likelihood(:,n-1);
            alpha(:,n) = logsumexp(log(T) + repmat(Joint(:,n-1),1,numHMMstates), 1)';
        end
        Joint(:,N) = alpha(:,N) + likelihood(:,N);
        
        for n = 1:N
            curLA = min(lookAhead, N-n);
            betaTmp = zeros(numHMMstates, curLA + 1);
            for l = 1:curLA
                betaTmp(:, end-l) = logsumexp(...
                    log(T) + ...
                    repmat(betaTmp(:, end-l+1)' + ...
                    likelihood(:, n+1+curLA-l)', numHMMstates, 1), 2);
            end
            beta(:,n) = betaTmp(:,1);
        end
        Joint = Joint + beta;
        
        %%% normalize
        Joint = exp(Joint - repmat(logsumexp(Joint,1), numHMMstates, 1));
        
        
        IDX = cell(numHMMstates,1);
        WEIGHTS = cell(numHMMstates,1);
        for t=1:size(Joint,2)
            [sortJ, sortIdx] = sort(Joint(:,t),'descend');
            tmpIdx = find(cumsum(sortJ) >= 0.99, 1, 'first');
            tmpW = sortJ(1:tmpIdx) / sum(sortJ(1:tmpIdx));
            
            for l=1:tmpIdx
                IDX{sortIdx(l)} = [IDX{sortIdx(l)}; t];
                WEIGHTS{sortIdx(l)} = [WEIGHTS{sortIdx(l)}; tmpW(l)];
            end
        end
        
        
        MPEData = zeros(size(Data));
        for kc = 1:numHMMstates
            
            idx = IDX{kc};
            weights = WEIGHTS{kc};
            if isempty(idx)
                continue
            end
            
            if SD
                modelname = sprintf('%sSPNHMM/Speaker%d/SPN_K%d_%d.mod', modelPath, speaker, numHMMstates, kc);
            else
                modelname = sprintf('%sSPNHMM/SpeakerIndependent/SPN_K%d_%d.mod', modelPath, numHMMstates, kc);
            end
            
            tmpMPE = inferMPE(Data(idx, :), [], modelname, binPath);
            MPEData(idx,:) = MPEData(idx,:) + tmpMPE .* repmat(weights, 1, size(MPEData,2));
        end
        
        MPEData(~isnan(Data)) = Data(~isnan(Data));
        MPEData = MPEData .* repmat(SIGMA, size(MPEData,1), 1);
        MPEData = MPEData + repmat(MU, size(MPEData,1), 1);
                
        XRec = exp(MPEData');
        XRec(1:2,:) = X(1:2,:);
        
        %%% construct phase, Lim Griffin
        phaseRec = zeros(size(XRec));
        phaseRec(1:125,:) = phase(1:125,:);
        
        clear E
        for c = 1:100
            Xtmp = XRec .* exp(1i*phaseRec);
            xtmp = ISTFT_correct_window(Xtmp, numOverlap, win);
            Xtmp = spectrogram(xtmp, win, numOverlap, frameLen);
            
            E(c) = sum(sum((XRec - abs(Xtmp)).^2));
            
            phaseRec = angle(Xtmp);
            phaseRec(1:125,:) = phase(1:125,:);
        end
        
        %figure(1)
        %clf
        %imagesc(flipud(log(abs(XRec))))
        
        %figure(2)
        %clf
        %imagesc(flipud(XTRUE))
        
        %%% ISTFT and save
        r = ISTFT_correct_window(XRec .* exp(1i*phaseRec), numOverlap, win);
        r = 0.999 * r / max(abs(r));
        
        audiowrite(fileName, r, fsWork, 'BitsPerSample', 24);
    end
end


