
setPaths

speakerLabels = [1,2,18,20];
numSpeakers = length(speakerLabels);
K = 64;

%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Speaker dependent %%%
%%%%%%%%%%%%%%%%%%%%%%%%%

for sCount = 1:numSpeakers
    speaker = speakerLabels(sCount);
    fprintf('Speaker: %d\n',speaker);
    
    clear C
    clear idx
    load(sprintf('%sClusterings/TrainSpeaker%d_%d.mat', dataPath, speaker, K));
    
    uniIdx = unique(idx);
    
    for i = uniIdx
        curPath = sprintf('%sSPNHMM/Speaker%d/', modelPath, speaker);
        [SUB, MSG] = mkdir(curPath);
        modelname = sprintf('%sSPNHMM/Speaker%d/SPN_K%d_%d.mod', modelPath, speaker, K, i);
        
        if exist(modelname, 'file')
            continue
        end
        
        tmp = 'tmp';
        save(modelname, 'tmp')
        
        load(sprintf('%sTrainSpeaker%d.mat', dataPath, speaker));
        Data = Data(idx == i,:);
        fprintf('   train SPN %d/%d, %d samples\n', i, max(uniIdx), size(Data,1));
                
        trainSPN_PD(Data, 1, 257, ...
            'binPath', binPath,...
            'numGauss', 20, ...
            'numSums', 20, ...
            'coarseRes', [], ...
            'coarseResX', [1], ...
            'coarseResY', [4], ...
            'modelFile', modelname,...
            'numIter', 3,...
            'batchSize', 1);
    end
    
    
    %%%
    numHMMstates = max(idx);
    
    %%% estimate priors
    P = hist(idx, 1:numHMMstates) + 1;
    P = P / sum(P);
    
    %%% estimate transition matrix
    T = zeros(numHMMstates, numHMMstates);
    for s = 1:numHMMstates
        curIdx = find(idx(1:end-1) == s);
        T(s,:) = hist(idx(curIdx+1), 1:numHMMstates) + 1;
    end
    sT = sum(T,2);
    T = T./repmat(sT,1,numHMMstates);
    
    
    modelname = sprintf('%sSPNHMM/Speaker%d/HMM_K%d.mat', modelPath, speaker, K);
    save(modelname, 'P', 'T');    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Speaker independent %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('Speaker independent\n');

clear C
clear idx
load(sprintf('%sClusterings/TrainSpeakerIndependent_%d.mat', dataPath, K));

uniIdx = unique(idx);

for i = uniIdx
    curPath = sprintf('%sSPNHMM/SpeakerIndependent/', modelPath);
    [SUB, MSG] = mkdir(curPath);
    modelname = sprintf('%sSPNHMM/SpeakerIndependent/SPN_K%d_%d.mod', modelPath, K, i);
    
    if exist(modelname, 'file')
        continue
    end
    
    tmp = 'tmp';
    save(modelname, 'tmp')
    
    load(sprintf('%sTrainSpeakerIndependent.mat', dataPath));
    Data = Data(idx == i,:);
    fprintf('   train SPN %d/%d, %d samples\n', i, max(uniIdx), size(Data,1));
    
    trainSPN_PD(Data, 1, 257, ...
        'binPath', binPath,...
        'numGauss', 20, ...
        'numSums', 20, ...
        'coarseRes', [], ...
        'coarseResX', [1], ...
        'coarseResY', [4], ...
        'modelFile', modelname,...
        'numIter', 3,...
        'batchSize', 1);
end

load([dataPath, 'PreprocessSpeakerIndependent.mat']);

%%% estimate priors
P = hist(idx, 1:K) + 1;
P = P / sum(P);

%%% estimate transition matrix
T = zeros(K, K);
si = 1;
for sCount = 1:length(numSamplesOfSpeaker)
    locIdx = idx(si:si+numSamplesOfSpeaker(sCount)-1);
    
    for s = 1:K
        curIdx = find(locIdx(1:end-1) == s);
        T(s,:) = T(s,:) + hist(locIdx(curIdx+1), 1:K);
    end
    
    si = si + numSamplesOfSpeaker(sCount);
end
T = T + 1;
sT = sum(T,2);
T = T./repmat(sT,1,K);

modelname = sprintf('%sSPNHMM/SpeakerIndependent/HMM_K%d.mat', modelPath, K);
save(modelname, 'P', 'T');


